import datetime as dt
from datetime import datetime, timedelta

from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.models import User
from django import forms
from django.shortcuts import render_to_response, get_object_or_404
from django.template import Context, loader, RequestContext
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.conf import settings
from django.template.loader import render_to_string

from pidge.groups.models import Group, GroupService, ServiceProperty, GroupServiceProperty, ParseLocation, GroupsUser, Membership
from basic.funcs import MustLogin, PidgeException, MustLoginMessage

from models import Event, EventTime, EventTag, EventsUser, Reminder
from forms import EventForm, AddTimeForm


def All(request):
    ''' List all events. '''
    from django.db.models import Q
    
    venue = None
    venue_gsp = None
    category = None
    loggedin = False
    allevents = None
    
    categoryid = 0
    if request.GET.get('category', False):
        try:
            categoryid = int(request.GET['category'])
        except ValueError:
            pass
            
    if 'loggedin' in request.session:
        user = User.objects.get(id = request.session['userid'])
        loggedin = True

    if categoryid == -1 or (categoryid == 0 and not loggedin):
        allevents = True
    
    events = EventTime.objects.filter(start__gte = datetime.now() - timedelta(hours = 1)).order_by('start');
            
    if loggedin and categoryid == 0:
        # Events includes any groups which the user is associated with
        # and those which fit his interests.
        
        events = events.filter(Q(event__group__membership__person = user) | Q(event__tags__user = user)).distinct()
    else:
        if categoryid > 0:
            try:
                category = EventTag.objects.get(id = request.REQUEST['category'])
                events = events.filter(event__tags__id = request.REQUEST['category'])
            except EventTag.DoesNotExist:
                pass

    search_query = ""
    if "q" in request.REQUEST:
        search_query = request.REQUEST['q']
        if len(search_query) > 0:
            events = events.filter(Q(event__title__icontains=search_query) | Q(event__description__icontains=search_query) | Q(event__group__official_name__icontains = search_query) | Q(event__group__friendly_name__icontains = search_query))
    
    if not category or not category.name == "Academic Events / Talks":
        events = events.exclude(event__group__friendly_name__exact = "talks.cam")

    tags = EventTag.objects.all().order_by('name')
    
    bc = [{'title': 'Overview'}]
    return render_to_response(
        "events-all",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'events': events,
            'venue': venue,
            'venuegsp': venue_gsp,
            'tags': tags,
            'category': category,
            'allevents': allevents,
            'loggedin': loggedin,
            'search_query': search_query,
            'categoryid': categoryid
        },
        context_instance = RequestContext(request)
    )

def RenderICS(eventtimes, filename):
    t = loader.get_template("events-ics")
    response = HttpResponse(t.render(Context({'eventtimes':eventtimes})), mimetype="text/calendar")
    response['Content-Disposition'] = 'attachment; filename=' + filename
    return response
    
def ViewEvent(request, eventid, format=""):
    ''' Show the details of an event. '''

    event = get_object_or_404(Event, id = eventid)
    
    flags = {}
    if request.user.is_authenticated():
        usr, created = EventsUser.objects.get_or_create(user = request.user)
        groups_user, created = GroupsUser.objects.get_or_create(user = request.user)
        

        try:
            uea_hour = Reminder.objects.get(events_user = usr, event = event, assoctype='EML_HR')
            flags['email_hour'] = True
        except Reminder.DoesNotExist:
            pass
        try:
            uea_day = Reminder.objects.get(events_user = usr, event = event, assoctype='EML_DY')
            flags['email_day'] = True
        except Reminder.DoesNotExist:
            pass 
        
        try:
            uga = Membership.objects.get(groups_user = groups_user, group=event.group, assoctype='CTL')
            flags['controller'] = True
        except Membership.DoesNotExist:
            pass
    
    location = ParseLocation(event.location_additional)
    
    eventtimes = EventTime.objects.filter(event=event).order_by('start')

    if format == "ics":
        filename = "event" + eventid + ".vcs"
        return RenderICS(eventtimes, filename)
    else:
        bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')}, {'title': event.title}]
        return render_to_response(
            "events-event",
            {
                'breadcrumb': bc,
                'activetab': 'Events',
                'event': event,
                'eventtimes': eventtimes,
                'flags': flags,
                'location_address': location['address'],
                'location_allowshow': location['allowshow'],
                'location_lati': location['lati'],
                'location_longi': location['longi'],
                'servername': settings.SERVER_NAME
            },
            context_instance = RequestContext(request)
        )


@MustLoginMessage("Please log in to add an event.")
def Add(request):
    ''' Show groups to which user can add an event / option to create group. '''
    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')}, {'title': 'Add Event'}]
        
    usr, created = GroupsUser.objects.get_or_create(user = request.user)
    ugas = Membership.objects.filter(groups_user = usr, assoctype='CTL')
    return render_to_response(
        "events-add",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'ugas': ugas
        },
        context_instance = RequestContext(request)
    )

@MustLoginMessage("Please log in to add an event.")
def AddOrClone(request, groupid):
    ''' Show choice: add event or clone existing. '''
    group = get_object_or_404(Group, id = groupid)
    usr, created = GroupsUser.objects.get_or_create(user = request.user)

    try:
        uga = Membership.objects.get(groups_user = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied
    
    events = Event.objects.filter(group = group)
    
    if events.count() == 0:
        return HttpResponseRedirect(reverse('pidge.events.views.AddToGroup', args = [groupid]))
    
    bc = [
        {'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': 'Add Event', 'url': reverse('pidge.events.views.Add')},
        {'title': group.official_name}  ]
    return render_to_response(
        "events-add-or-clone",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'events': events,
            'group': group
        },
        context_instance = RequestContext(request)
    )


@MustLoginMessage("Please log in to add an event.")
def AddToGroup(request, groupid):
    ''' Add an event to a specific group. '''
    
    usr, created = GroupsUser.objects.get_or_create(user = request.user)
    group = get_object_or_404(Group, id = groupid)

    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': 'Add Event', 'url': reverse('pidge.events.views.Add')},
        {'title': group.official_name}]

    try:
        uga = Membership.objects.get(groups_user = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied
    
    eventClone = None
    # Does the user want to clone an existing Event for this Group?
    if 'clone' in request.REQUEST:
        eventClone = get_object_or_404(Event, id=request.REQUEST['clone'])
        
    if request.method == 'POST':
        myevent = Event(group = group)
        aef = EventForm(request.POST, instance = myevent)
        if aef.is_valid():
            # Save new Event and get them to add EventTimes to it
            newevent = aef.save()
            return HttpResponseRedirect(reverse('pidge.events.views.AddTime', args=[newevent.id]))
        
    else:
        if eventClone:
            aef = EventForm(instance = eventClone)
        else:
            aef = EventForm()
    
    return render_to_response(
        "events-add-togroup",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'group': group,
            'aef': aef
        },
        context_instance = RequestContext(request)
    )



@MustLoginMessage("Please log in to edit this event.")
def EditEvent(request, eventid):
    ''' Show and update existing event. '''
    
    event = get_object_or_404(Event, id = eventid)
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': event.title, 'url': reverse('pidge.events.views.ViewEvent', args=[event.id])},
        {'title':'Edit'}]
    
    usr, created = GroupsUser.objects.get_or_create(user = request.user)
    
    try:
        uga = Membership.objects.get(groups_user = usr, group=group, assoctype='CTL')
    except(Membership.DoesNotExist):
        raise PermissionDenied
    
    if request.method == 'POST':
        editform = EventForm(request.POST, instance = event)
        if editform.is_valid():
            editform.save()
            return HttpResponseRedirect(reverse('pidge.events.views.ViewEvent', args=[event.id]))
    else:
        editform = EventForm(instance = event)
    return render_to_response(
        "events-edit",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'event': event,
            'editform': editform
        },
        context_instance = RequestContext(request)
    )


@MustLoginMessage("Please log in to delete this time.")
def DeleteTime(request, eventid, eventtimeid):
    ''' Delete an EventTime for an Event. '''

    event = get_object_or_404(Event, id = eventid)
    eventtime = get_object_or_404(EventTime, id = eventtimeid)    
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': event.title, 'url': reverse('pidge.events.views.ViewEvent', args=[event.id])},
        {'title':'Add Time'}]
        
    usr, created = GroupsUser.objects.get_or_create(user = request.user)
    
    try:
        uga = Membership.objects.get(groups_user = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied

    if request.method == 'POST':
        eventtime.delete()
        return HttpResponseRedirect(reverse('pidge.events.views.ViewEvent', args=[event.id]))
    
    return render_to_response(
        "events-delete-time",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'event': event,
            'eventtime': eventtime
        },
        context_instance = RequestContext(request)
    )

def NowAddTime(request, eventid):
    ''' After adding an event, add times '''
    return AddTime(request, eventid, eventadded = True)

@MustLoginMessage("Please log in to add a time to this event.")
def AddTime(request, eventid, eventadded = False):
    ''' Show form to add EventTime to Event. '''
    flags = {}

    event = get_object_or_404(Event, id = eventid)
    
    if eventadded:
        flags['event_added'] = True
        
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': event.title, 'url': reverse('pidge.events.views.ViewEvent', args=[unicode(event.id)])},
        {'title':'Add Time'}]
        
    usr, created = GroupsUser.objects.get_or_create(user = request.user)
    
    try:
        uga = Membership.objects.get(groups_user = usr, group=group, assoctype='CTL')
    except(Membership.DoesNotExist):
        raise PermissionDenied


    if request.method == 'POST':
        myeventtime = EventTime(event = event)
        atf = AddTimeForm(request.POST, instance = myeventtime)
        if atf.is_valid():
            atf.save()
            # Now reset the form so they can add another
            atf = AddTimeForm()
            flags['added'] = True
            
    else:
        atf= AddTimeForm()

    return render_to_response(
        "events-add-time",
        {
            'breadcrumb': bc,
            'activetab': 'Events',
            'event': event,
            'atf': atf,
            'MEDIA_URL': settings.MEDIA_URL,
            'flags': flags
        },
        context_instance = RequestContext(request)
    )



@MustLoginMessage("You must be logged in to add an event reminder, so that we know who to email!.")
def SetReminders(request, eventid):
    ''' Update user's reminders for this Event. '''
    event = get_object_or_404(Event, id = eventid)
    
    bc = [{'title': 'Overview', 'url': reverse('pidge.events.views.All')},
        {'title': event.title, 'url': reverse('pidge.events.views.ViewEvent', args=[event.id])},
        {'title': 'Set Reminders'}]
    
    usr, created = EventsUser.objects.get_or_create(user = request.user)
    
    if 'hour' in request.GET and request.GET['hour'] == '1':
        Reminder.objects.get_or_create(
            events_user = usr,
            event = event,
            assoctype = 'EML_HR'
        )
    else:
        Reminder.objects.filter(events_user = usr, event = event, assoctype = 'EML_HR').delete()
    
    if 'day' in request.GET and request.GET['day'] == '1':
        new_reminder = Reminder.objects.get_or_create(events_user = usr, event=event, assoctype='EML_DY')
    else:
        Reminder.objects.filter(events_user = usr, event=event, assoctype='EML_DY').delete()
    
    return HttpResponseRedirect(reverse('pidge.events.views.ViewEvent', args=[unicode(event.id)]))

